!--------------------------------------------------------------------------
! FOLLOWER: an Inform library by Andrew Clover
! Based on Gareth Rees's original code.
! Updated for Inform 5.5 by Gareth Rees.
! 
! An Inform library to allow things to be FOLLOWed. To use, `Include' it
! after `verblib' and before `grammar', then give any object that can be
! followed `class followclass'. To move a followable object, call
! MoveNPC(object, destination, actiontomove, objectnumber). Pass 0 for
! actionnumber and objectnumber to disallow following.  Pass 0 for
! destination to remove object.
! 
! When the player attempts to follow the object, the action <actiontomove
! objectnumber> will be generated.
! 
! Examples: MoveNPC(bob, bobshouse, ##Go, n_obj);
!           MoveNPC(driver, car, ##Enter, car);
!--------------------------------------------------------------------------

Property follow_action;
Property follow_object;
Attribute followable;

[ NewRoom i;
    for (i = selfobj + 1: i <= top_object: i++)
        if (i has followable)
            i.follow_action=0;
    rfalse;
];

[ FollowScope i;
    if (scope_stage==1) rfalse;
    if (scope_stage==2) {
        for (i = selfobj + 1: i <= top_object: i++)
            if (i has followable && i.follow_action ~= 0)
                PlaceInScope(i);
        rfalse;
    }
    "Go where?";
];

[ FollowSub oldnoun oldpos;
    if (parent(noun) == location or player)
        print_ret "You are already with ", (the) noun, ".";
    if (noun hasnt followable)
        "That's not something you can follow.";
    if (noun.follow_action == 0) {
        print "You don't know where ";
        if (noun has animate) {
            if (noun has female) print "she"; else print "he";
        } else print "that";
        " is.";
    }
    oldnoun = noun;
    oldpos = parent(player);
    <(noun.follow_action) noun.follow_object>;
    if (oldnoun notin parent(player) && parent(player) ~= oldpos)
        print "^(", (the) oldnoun, " doesn't seem to be here any more.)^";
    rtrue;
];

[ MoveNPC object dest actn objn;
    if (object has followable) {
        if (dest == parent(player))
            object.follow_action = 0;
        else
            if (parent(object) == parent(player)) {
                object.follow_action = actn;
                object.follow_object = objn;
            }
    }
    if (dest==0) remove object;
    else move object to dest;
];

Class   followclass
 has    followable,
 with   follow_action 0,
        follow_object 0;

Verb "follow" "chase" "pursue" "trail"
    * scope=FollowScope -> Follow;
