!--------------------------------------------------------------------------
! PLURALOBJ: an Inform library
! 
! (c) Andrew Clover, 1995, but freely usable. Release 1.
! Updated to Inform 5.5 by Gareth Rees.
! 
! An Inform library to allow plural nouns. For example, a pair of gloves,
! some adverts, or a pile of magazines. Give an object the attribute
! 'pluralobj' and the correct messages will appear.  Include it just before
! VerbLib.
! 
! This library provides LibraryMessages in order to change the library
! responses to plural-aware ones. If you want to provide your own library
! messages, call the LibraryMessages object LibraryMessages2. This version
! replaces all necessary messages for release 5.9 of the Inform library.
! 
! This library gives you some commands to use instead of just printing 'it'
! or such like. They are itthey, itthem, doesntdont, isntarent, thatthose,
! thisthese (lower case first letters); ittheyc, itstheyrec, thatthosec,
! thisthesec, thatstheyre, thattheyc (upper case). Also verbsuf, which just
! prints 's' for singular-form verbs. The commands also print the correct
! pronoun for animates. One parameter is necessary, an object number to
! check for pluralobj. With anything else, your printing commands will have
! to check for the pluralobj flag themselves.
!--------------------------------------------------------------------------

Attribute pluralobj;

Object  LibraryMessages "lm"
 with   before [ i;
            #IFDEF LibraryMessages2;
            i = runroutines(LibraryMessages2, before);
            if (i ~= 0) return i;
            #ENDIF;
         Take:
            switch (lm_n) {
             5: print_ret "You already have ", (thatthose) noun, ".";
             6: print_ret (thatthosec) noun, " seem", (verbsuf) noun,
                    " to belong to ", (the) lm_o, ".";
             7: print_ret (thatthosec) noun, " seem", (verbsuf) noun,
                    " to be a part of ", (the) lm_o, ".";
             8: print_ret (thatthosec) noun, " ", (thatthosec) noun,
                    " ", (isntarent) noun, " available.";
             9: print_ret (The) lm_o, " ", (isntarent) lm_o, " open.";
            10: print_ret (thatstheyrec) noun, " hardly portable.";
            11: print_ret (thatstheyrec) noun, " fixed in place.";
            }
         Drop:
            switch (lm_n) {
             2: print_ret "You haven't got ", (thatthose) noun, ".";
            }
         Remove:
            switch (lm_n) {
             2: print_ret "But ", (itthey) noun, " ", (isntarent) noun, 
                    " there now.";
             3: print_ret "You'll need to take ", (itthem) noun, " off first.";
            }
         PutOn:
            switch (lm_n) {
             1: print_ret "You need to be holding ", (itthem) noun, 
                    " before you can put ", (itthem) noun, " on top of \
                    something else.";
             5: print_ret "(first taking ", (itthem) noun, " off)^";
            }
         Insert:
            switch (lm_n) {
             1: print_ret "You need to be holding ", (itthem) noun, 
                    " before you can put ", (itthem) noun, " into \
                    something else.";
             2: print_ret (thattheyc) second, " can't contain things.";
             3: print_ret "Alas, ", (itthey) second, " ", (isare) second, 
                    " closed.";
             4: print_ret "You'll need to take ", (itthem) second, 
                    " off first.";
             6: print_ret "(first taking ", (itthem) noun, " off)^";
            }
         Transfer:
            switch (lm_n) {
             1: print_ret (thatthose) noun, " ", (isntarent) noun,
                    " in your possession.";
             2: print_ret "First pick ", (itthem) noun, " up.";
            }
         Enter:
            switch (lm_n) {
             2: print_ret (thatstheyrec) noun, " not something you can enter.";
            }
         Search:
            switch (lm_n) {
             5: print_ret "You can't see inside, since ", (itthey) noun, 
                    " ", (isare) noun, " closed.";
             6: print_ret (The) lm_o, " ", (isare) lm_o, " closed.";
            }
         Unlock:
            switch (lm_n) {
             1: print_ret (thattheyc) noun, " ", (doesntdont) noun,
                    " seem to be something you can unlock.";
             2: print_ret (itstheyrec) noun, " unlocked at the moment.";
             3: print_ret (thatthosec) second, " ", (doesntdont) second, 
                    " seem to fit the lock.";
            }
         Lock:
            switch (lm_n) {
             1: print_ret (thattheyc) noun, " ", (doesntdont) noun, 
                    " seem to be something you can lock.";
             2: print_ret (itstheyrec) noun, " locked at the moment.";
             3: print_ret "First you'll have to close ", (itthem) noun, ".";
             4: print_ret thatthose(second), " ", doesntdont(second), 
                    " seem to fit the lock.";
            }
         SwitchOn:
            switch (lm_n) {
             1: print_ret (thatstheyrec) noun, " not something you can \
                    switch.",
             2: print_ret (thatstheyrec) noun, " already on.";
            }
         SwitchOff:
            switch (lm_n) {
             1: print_ret (thatstheyrec) noun, " not something you can \
                    switch.";
             2: print_ret (thatstheyrec) noun, " already off.";
            }
         Open:
            switch (lm_n) {
             1: print_ret (thatstheyrec) noun, " not something you can open.";
             2: print_ret (ittheyc) noun, " seem", (verbsuf) noun, 
                    " to be locked.";
             3: print_ret (itstheyrec) noun, " already open.",
            }
         Close:
            switch (lm_n) {
             1: print_ret (thatstheyrec) noun, " not something you can close.";
             2: print_ret (itstheyrec) noun, " already closed.";
            }
         Disrobe:
            switch (lm_n) {
             1: print_ret "You're not wearing ", (thatthose) noun, ".";
            }
         Wear:
            switch (lm_n) {
             1: print_ret "You can't wear ", (thatthose) noun, "!";
             2: print_ret "You're not holding ", (itthem) noun, ".";
             3: print_ret "You're already wearing ", (thatthose) noun, "!";
            }
         Eat:
            switch (lm_n) {
             1: print_ret (thatstheyrec) noun, " plainly inedible.";
            }
         Cut:
            print_ret "Cutting ", (thatthose) noun, " would achieve little.";
         Blow:
            print_ret "You can't usefully blow ", (thatthose) noun, ".";
         Set:
            print_ret "No, you can't set ", (thatthose) noun, ".";
         SetTo:
            print_ret "No, you can't set ", (thatthose) noun, " to anything.";
         Wave:
            switch (lm_n) {
             1: print_ret "But you aren't holding ", (thatthose) noun, ".";
            }
         Push, Pull, Turn:
            switch (lm_n) {
             1: print_ret (itstheyrec) noun, " fixed in place.";
            }
         0: i=action;  ! pointless statements to stop a compiler warning
                       ! 'unused variable' appearing if lm2 object absent
       ];

[ PrintPronoun i j k;
    if (i has pluralobj) k = 0;
    else {
        if (i has animate) {
            if (i has female) k = 1;
            else k = 2;
        } else k = 3;
    }
    print (string) (PronounArray-->(j * 4 + k));
];

Array PronounArray --> [;
    "they" "she" "he" "it";
    "They" "She" "He" "That";
    "them" "her" "him" "it";
    "They" "She" "He" "It";
    "They're" "She's" "He's" "It's";
    "They're" "She's" "He's" "That's";
];

[ itthey i; PrintPronoun(i,0); ];
[ thattheyc i; PrintPronoun(i,1); ];
[ itthem i; PrintPronoun(i,2); ];
[ ittheyc i; PrintPronoun(i,3); ];
[ itstheyrec i; PrintPronoun(i,4); ];
[ thatstheyrec i; PrintPronoun(i,5); ];

[ doesntdont i; if (i has pluralobj) print "don't";  else print "doesn't"; ];
[ isntarent i;  if (i has pluralobj) print "aren't"; else print "isn't";   ];
[ thatthose i;  if (i has pluralobj) print "those";  else print "that";    ];
[ thatthosec i; if (i has pluralobj) print "Those";  else print "That";    ];
[ thisthese i;  if (i has pluralobj) print "these";  else print "this";    ];
[ thisthesec i; if (i has pluralobj) print "These";  else print "This";    ];
[ isare i;      if (i has pluralobj) print "are";    else print "is";      ];
[ verbsuf i;    if (i hasnt pluralobj) print "s"; ];
